@@ -1,16 +1,16 @@
-# Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
 # published by the Free Software Foundation.
 #
-# This program is also distributed with certain software (including
+# This program is designed to work with certain software (including
 # but not limited to OpenSSL) that is licensed under separate terms,
 # as designated in a particular file or component or in included license
-# documentation.  The authors of MySQL hereby grant you an
+# documentation. The authors of MySQL hereby grant you an
 # additional permission to link the program and your derivative works
-# with the separately licensed software that they have included with
-# MySQL.
+# with the separately licensed software that they have either included with
+# the program or referenced in the documentation.
 #
 # Without limiting anything contained in the foregoing, this file,
 # which is part of MySQL Connector/Python, is also subject to the
@@ -28,59 +28,53 @@
 
 """Implementation of the Result classes."""
 
+from __future__ import annotations
+
 import decimal
 import struct
 import sys
 
 from datetime import datetime, timedelta
+from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
 
-from .dbdoc import DbDoc
 from .charsets import MYSQL_CHARACTER_SETS
-from .compat import STRING_TYPES
+from .dbdoc import DbDoc
 from .helpers import decode_from_bytes, deprecated
+from .types import ConnectionType, FieldTypes
 
 
-# pylint: disable=C0111
-def from_protobuf(col_type, payload):
+# pylint: disable=missing-class-docstring,missing-function-docstring
+def from_protobuf(column: Column, payload: bytes) -> Any:
     if len(payload) == 0:
         return None
 
+    if column.get_type() == ColumnType.STRING:
+        return decode_from_bytes(payload[:-1])  # Strip trailing char
+
     try:
-        return {
-            ColumnProtoType.SINT: varsint_from_protobuf,
-            ColumnProtoType.UINT: varint_from_protobuf,
-            ColumnProtoType.BYTES: bytes_from_protobuf,
-            ColumnProtoType.DATETIME: datetime_from_protobuf,
-            ColumnProtoType.TIME: time_from_protobuf,
-            ColumnProtoType.FLOAT: float_from_protobuf,
-            ColumnProtoType.DOUBLE: double_from_protobuf,
-            ColumnProtoType.BIT: varint_from_protobuf,
-            ColumnProtoType.SET: set_from_protobuf,
-            ColumnProtoType.ENUM: bytes_from_protobuf,
-            ColumnProtoType.DECIMAL: decimal_from_protobuf,
-        }[col_type](payload)
+        return ColumnProtoType.converter_map[column.get_proto_type()](payload)
     except KeyError as err:
-        sys.stderr.write("{0}".format(err))
-        sys.stderr.write("{0}".format(payload.encode("hex")))
+        sys.stderr.write(f"{err}")
+        sys.stderr.write(f"{payload.encode('hex')}")  # type: ignore[attr-defined]
         return None
 
 
-def bytes_from_protobuf(payload):
+def bytes_from_protobuf(payload: bytes) -> bytes:
     # Strip trailing char
-    return decode_from_bytes(payload[:-1])
+    return payload[:-1]
 
 
-def float_from_protobuf(payload):
+def float_from_protobuf(payload: bytes) -> float:
     assert len(payload) == 4
-    return struct.unpack("<f", payload)
+    return struct.unpack("<f", payload)[0]
 
 
-def double_from_protobuf(payload):
+def double_from_protobuf(payload: bytes) -> float:
     assert len(payload) == 8
-    return struct.unpack("<d", payload)
+    return struct.unpack("<d", payload)[0]
 
 
-def varint_from_protobuf_stream(payload):
+def varint_from_protobuf_stream(payload: bytes) -> Tuple[int, bytes]:
     if len(payload) == 0:
         raise ValueError("Payload is empty")
 
@@ -91,18 +85,18 @@ def varint_from_protobuf_stream(payload):
     for item in payload:
         char = item if isinstance(item, int) else ord(item)
         eos = (char & 0x80) == 0
-        cur_bits = (char & 0x7f)
+        cur_bits = char & 0x7F
         cur_bits <<= shift
         i |= cur_bits
         if eos:
-            return i, payload[cur + 1:]
+            return i, payload[cur + 1 :]
         cur += 1
         shift += 7
 
     raise EOFError("Payload too short")
 
 
-def varint_from_protobuf(payload):
+def varint_from_protobuf(payload: bytes) -> int:
     i, payload = varint_from_protobuf_stream(payload)
     if len(payload) != 0:
         raise ValueError("Payload too long")
@@ -110,7 +104,7 @@ def varint_from_protobuf(payload):
     return i
 
 
-def varsint_from_protobuf(payload):
+def varsint_from_protobuf(payload: bytes) -> int:
     i, payload = varint_from_protobuf_stream(payload)
     if len(payload) != 0:
         raise ValueError("Payload too long")
@@ -118,16 +112,16 @@ def varsint_from_protobuf(payload):
     # Zigzag encoded, revert it
     if i & 0x1:
         i = ~i
-        i = (i >> 1)
+        i = i >> 1
         i |= 1 << 63
     else:
-        i = (i >> 1)
+        i = i >> 1
 
     return i
 
 
-def set_from_protobuf(payload):
-    set_pb = []
+def set_from_protobuf(payload: bytes) -> List[bytes]:
+    set_pb: List = []
     while True:
         try:
             field_len, payload = varint_from_protobuf_stream(payload)
@@ -147,7 +141,7 @@ def set_from_protobuf(payload):
     return set_pb
 
 
-def decimal_from_protobuf(payload):
+def decimal_from_protobuf(payload: bytes) -> decimal.Decimal:
     digits = []
     sign = None
     scale = payload[0] if isinstance(payload[0], int) else ord(payload[0])
@@ -155,35 +149,35 @@ def decimal_from_protobuf(payload):
 
     for item in payload:
         char = item if isinstance(item, int) else ord(item)
-        high_bcd = (char & 0xf0) >> 4
-        low_bcd = char & 0x0f
-        if high_bcd < 0x0a:
+        high_bcd = (char & 0xF0) >> 4
+        low_bcd = char & 0x0F
+        if high_bcd < 0x0A:
             digits.append(high_bcd)
-            if low_bcd < 0x0a:
+            if low_bcd < 0x0A:
                 digits.append(low_bcd)
-            elif low_bcd == 0x0c:
+            elif low_bcd == 0x0C:
                 sign = 0
                 break
-            elif low_bcd == 0x0d:
+            elif low_bcd == 0x0D:
                 sign = 1
                 break
             else:
                 raise ValueError("Invalid BCD")
-        elif high_bcd == 0x0c:
+        elif high_bcd == 0x0C:
             sign = 0
             assert low_bcd == 0x00
             break
-        elif high_bcd == 0x0d:
+        elif high_bcd == 0x0D:
             sign = 1
             assert low_bcd == 0x00
             break
         else:
-            raise ValueError("Invalid BCD: {0}".format(high_bcd))
+            raise ValueError(f"Invalid BCD: {high_bcd}")
 
     return decimal.Decimal((sign, digits, -scale))
 
 
-def datetime_from_protobuf(payload):
+def datetime_from_protobuf(payload: bytes) -> datetime:
     # A sequence of varints
     hour = 0
     minutes = 0
@@ -204,7 +198,7 @@ def datetime_from_protobuf(payload):
     return datetime(year, month, day, hour, minutes, seconds, useconds)
 
 
-def time_from_protobuf(payload):
+def time_from_protobuf(payload: bytes) -> timedelta:
     # A sequence of varints
     hour = 0
     minutes = 0
@@ -232,15 +226,16 @@ def time_from_protobuf(payload):
         elif useconds:
             useconds *= -1
 
-    return timedelta(hours=hour, minutes=minutes, seconds=seconds,
-                     microseconds=useconds)
+    return timedelta(
+        hours=hour, minutes=minutes, seconds=seconds, microseconds=useconds
+    )
 
 
-class Collations(object):
+class Collations:
     UTF8_GENERAL_CI = 33
 
 
-class ColumnType(object):
+class ColumnType:
     BIT = 1
     TINYINT = 2
     SMALLINT = 3
@@ -278,45 +273,69 @@ class ColumnType(object):
     LONGTEXT = 35
 
     @classmethod
-    def to_string(cls, needle):
+    def to_string(cls, needle: Any) -> Optional[str]:
         for key, value in vars(cls).items():
             if value == needle:
                 return key
         return None
 
     @classmethod
-    def from_string(cls, key):
+    def from_string(cls, key: str) -> Any:
         return getattr(cls, key.upper(), None)
 
     @classmethod
-    def is_char(cls, col_type):
-        return col_type in (cls.CHAR, cls.VARCHAR,)
+    def is_char(cls, col_type: int) -> bool:
+        return col_type in (
+            cls.CHAR,
+            cls.VARCHAR,
+        )
 
     @classmethod
-    def is_binary(cls, col_type):
-        return col_type in (cls.BINARY, cls.VARBINARY,)
+    def is_binary(cls, col_type: int) -> bool:
+        return col_type in (
+            cls.BINARY,
+            cls.VARBINARY,
+        )
 
     @classmethod
-    def is_text(cls, col_type):
-        return col_type in (cls.TEXT, cls.TINYTEXT, cls.MEDIUMTEXT,
-                            cls.LONGTEXT,)
+    def is_text(cls, col_type: int) -> bool:
+        return col_type in (
+            cls.TEXT,
+            cls.TINYTEXT,
+            cls.MEDIUMTEXT,
+            cls.LONGTEXT,
+        )
 
     @classmethod
-    def is_decimals(cls, col_type):
-        return col_type in (cls.REAL, cls.DOUBLE, cls.FLOAT, cls.DECIMAL,
-                            cls.NUMERIC,)
+    def is_decimals(cls, col_type: int) -> bool:
+        return col_type in (
+            cls.REAL,
+            cls.DOUBLE,
+            cls.FLOAT,
+            cls.DECIMAL,
+            cls.NUMERIC,
+        )
 
     @classmethod
-    def is_numeric(cls, col_type):
-        return col_type in (cls.BIT, cls.TINYINT, cls.SMALLINT, cls.MEDIUMINT,
-                            cls.INT, cls.BIGINT,)
+    def is_numeric(cls, col_type: int) -> bool:
+        return col_type in (
+            cls.BIT,
+            cls.TINYINT,
+            cls.SMALLINT,
+            cls.MEDIUMINT,
+            cls.INT,
+            cls.BIGINT,
+        )
 
     @classmethod
-    def is_finite_set(cls, col_type):
-        return col_type in (cls.SET, cls.ENUM,)
+    def is_finite_set(cls, col_type: int) -> bool:
+        return col_type in (
+            cls.SET,
+            cls.ENUM,
+        )
 
 
-class ColumnProtoType(object):
+class ColumnProtoType:
     SINT = 1
     UINT = 2
     DOUBLE = 5
@@ -329,20 +348,34 @@ class ColumnProtoType(object):
     BIT = 17
     DECIMAL = 18
 
+    converter_map: Dict[int, Callable[[bytes], Any]] = {
+        SINT: varsint_from_protobuf,
+        UINT: varint_from_protobuf,
+        BYTES: bytes_from_protobuf,
+        DATETIME: datetime_from_protobuf,
+        TIME: time_from_protobuf,
+        FLOAT: float_from_protobuf,
+        DOUBLE: double_from_protobuf,
+        BIT: varint_from_protobuf,
+        SET: set_from_protobuf,
+        ENUM: bytes_from_protobuf,
+        DECIMAL: decimal_from_protobuf,
+    }
 
-class Flags(object):
-    def __init__(self, value):
-        self._allowed_flags = {}
-        self._flag_names = {}
+
+class Flags:
+    def __init__(self, value: int) -> None:
+        self._allowed_flags: Dict[str, int] = {}
+        self._flag_names: Dict[int, str] = {}
         for key, val in self.__class__.__dict__.items():
             if key.startswith("__"):
                 continue
             if isinstance(val, int):
                 self._allowed_flags[key] = val
                 self._flag_names[val] = key
-        self._value = value
+        self._value: int = value
 
-    def __str__(self):
+    def __str__(self) -> str:
         mask = 1
         flag_names = []
         value = self._value
@@ -355,17 +388,17 @@ class Flags(object):
                 try:
                     flag_names.append(self._flag_names[flag])
                 except KeyError:
-                    sys.stderr.write("{0}".format(self._flag_names))
-                    sys.stderr.write("{0}".format(self.__class__.__dict__))
+                    sys.stderr.write(f"{self._flag_names}")
+                    sys.stderr.write(f"{self.__class__.__dict__}")
 
         return ",".join(flag_names)
 
     @property
-    def value(self):
+    def value(self) -> int:
         return self._value
 
     @value.setter
-    def value(self, val):
+    def value(self, val: int) -> None:
         self._value = val
 
 
@@ -401,10 +434,12 @@ class BytesContentType(ColumnFlags):
     GEOMETRY = 0x0001
     JSON = 0x0002
     XML = 0x0003
-# pylint: enable=C0111
 
 
-class Column(object):
+# pylint: enable=missing-class-docstring,missing-function-docstring
+
+
+class Column:
     """Represents meta data for a table column.
 
     Args:
@@ -423,54 +458,72 @@ class Column(object):
 
     .. versionchanged:: 8.0.12
     """
-    def __init__(self, col_type, catalog=None, schema=None, table=None,
-                 original_table=None, name=None, original_name=None,
-                 length=None, collation=None, fractional_digits=None,
-                 flags=None, content_type=None):
-        self._schema = decode_from_bytes(schema)
-        self._name = decode_from_bytes(name)
-        self._original_name = decode_from_bytes(original_name)
-        self._table = decode_from_bytes(table)
-        self._original_table = decode_from_bytes(original_table)
-        self._proto_type = col_type
-        self._col_type = None
-        self._catalog = catalog
-        self._length = length
-        self._collation = collation
-        self._fractional_digits = fractional_digits
-        self._flags = flags
-        self._content_type = content_type
-        self._number_signed = False
-        self._is_padded = False
-        self._is_binary = False
-        self._is_bytes = False
-        self._collation_name = None
-        self._character_set_name = None
-        self._zero_fill = None
+
+    def __init__(
+        self,
+        col_type: int,
+        catalog: Optional[str] = None,
+        schema: Optional[str] = None,
+        table: Optional[str] = None,
+        original_table: Optional[str] = None,
+        name: Optional[str] = None,
+        original_name: Optional[str] = None,
+        length: Optional[int] = None,
+        collation: Optional[int] = None,
+        fractional_digits: Optional[int] = None,
+        flags: Optional[int] = None,
+        content_type: Optional[int] = None,
+    ) -> None:
+        self._schema: str = decode_from_bytes(schema)
+        self._name: str = decode_from_bytes(name)
+        self._original_name: str = decode_from_bytes(original_name)
+        self._table: str = decode_from_bytes(table)
+        self._original_table: str = decode_from_bytes(original_table)
+        self._proto_type: int = col_type
+        self._col_type: Optional[int] = None
+        self._catalog: Optional[str] = catalog
+        self._length: Optional[int] = length
+        self._collation: Optional[int] = collation
+        self._fractional_digits: Optional[int] = fractional_digits
+        self._flags: Optional[int] = flags
+        self._content_type: Optional[int] = content_type
+        self._number_signed: bool = False
+        self._is_padded: Union[bool, int] = False
+        self._is_binary: bool = False
+        self._is_bytes: bool = False
+        self._collation_name: Optional[str] = None
+        self._character_set_name: Optional[str] = None
+        self._zero_fill: Optional[int] = None
 
         if self._collation > 0:
             if self._collation >= len(MYSQL_CHARACTER_SETS):
-                raise ValueError("No mapping found for collation {0}"
-                                 "".format(self._collation))
+                raise ValueError(f"No mapping found for collation {self._collation}")
             info = MYSQL_CHARACTER_SETS[self._collation]
             self._character_set_name = info[0]
             self._collation_name = info[1]
-            self._is_binary = ("binary" in self._collation_name or
-                               "_bin" in self._collation_name)
+            self._is_binary = (
+                "binary" in self._collation_name or "_bin" in self._collation_name
+            )
         self._map_type()
         self._is_bytes = self._col_type in (
-            ColumnType.GEOMETRY, ColumnType.JSON, ColumnType.XML,
-            ColumnType.BYTES, ColumnType.STRING)
+            ColumnType.GEOMETRY,
+            ColumnType.JSON,
+            ColumnType.XML,
+            ColumnType.BYTES,
+            ColumnType.STRING,
+        )
 
-    def __str__(self):
-        return str({
-            "col_type": self._col_type,
-            "schema": self._schema,
-            "table": self._table,
-            "flags": str(self._flags),
-        })
+    def __str__(self) -> str:
+        return str(
+            {
+                "col_type": self._col_type,
+                "schema": self._schema,
+                "table": self._table,
+                "flags": str(self._flags),
+            }
+        )
 
-    def _map_bytes(self):
+    def _map_bytes(self) -> None:
         """Map bytes."""
         if self._content_type == BytesContentType.GEOMETRY:
             self._col_type = ColumnType.GEOMETRY
@@ -484,7 +537,7 @@ class Column(object):
             self._col_type = ColumnType.STRING
         self._is_padded = self._flags & 1
 
-    def _map_datetime(self):
+    def _map_datetime(self) -> None:
         """Map datetime."""
         if self._length == 10:
             self._col_type = ColumnType.DATE
@@ -495,7 +548,7 @@ class Column(object):
         else:
             raise ValueError("Datetime mapping scenario unhandled")
 
-    def _map_int_type(self):
+    def _map_int_type(self) -> None:
         """Map int type."""
         if self._length <= 4:
             self._col_type = ColumnType.TINYINT
@@ -509,7 +562,7 @@ class Column(object):
             self._col_type = ColumnType.BIGINT
         self._number_signed = True
 
-    def _map_uint_type(self):
+    def _map_uint_type(self) -> None:
         """Map uint type."""
         if self._length <= 3:
             self._col_type = ColumnType.TINYINT
@@ -523,7 +576,7 @@ class Column(object):
             self._col_type = ColumnType.BIGINT
         self._zero_fill = self._flags & 1
 
-    def _map_type(self):
+    def _map_type(self) -> None:
         """Map type."""
         if self._proto_type == ColumnProtoType.SINT:
             self._map_int_type()
@@ -531,16 +584,13 @@ class Column(object):
             self._map_uint_type()
         elif self._proto_type == ColumnProtoType.FLOAT:
             self._col_type = ColumnType.FLOAT
-            self._is_number_signed = \
-                (self._flags & FloatColumnFlags.UNSIGNED) == 0
+            self._is_number_signed = (self._flags & FloatColumnFlags.UNSIGNED) == 0
         elif self._proto_type == ColumnProtoType.DECIMAL:
             self._col_type = ColumnType.DECIMAL
-            self._is_number_signed = \
-                (self._flags & FloatColumnFlags.UNSIGNED) == 0
+            self._is_number_signed = (self._flags & FloatColumnFlags.UNSIGNED) == 0
         elif self._proto_type == ColumnProtoType.DOUBLE:
             self._col_type = ColumnType.DOUBLE
-            self._is_number_signed = \
-                (self._flags & FloatColumnFlags.UNSIGNED) == 0
+            self._is_number_signed = (self._flags & FloatColumnFlags.UNSIGNED) == 0
         elif self._proto_type == ColumnProtoType.BYTES:
             self._map_bytes()
         elif self._proto_type == ColumnProtoType.TIME:
@@ -554,10 +604,10 @@ class Column(object):
         elif self._proto_type == ColumnProtoType.BIT:
             self._col_type = ColumnType.BIT
         else:
-            raise ValueError("Unknown column type {0}".format(self._proto_type))
+            raise ValueError(f"Unknown column type {self._proto_type}")
 
     @property
-    def schema_name(self):
+    def schema_name(self) -> str:
         """str: The schema name.
 
         .. versionadded:: 8.0.12
@@ -565,7 +615,7 @@ class Column(object):
         return self._schema
 
     @property
-    def table_name(self):
+    def table_name(self) -> str:
         """str: The table name.
 
         .. versionadded:: 8.0.12
@@ -573,7 +623,7 @@ class Column(object):
         return self._original_table or self._table
 
     @property
-    def table_label(self):
+    def table_label(self) -> str:
         """str: The table label.
 
         .. versionadded:: 8.0.12
@@ -581,7 +631,7 @@ class Column(object):
         return self._table or self._original_table
 
     @property
-    def column_name(self):
+    def column_name(self) -> str:
         """str: The column name.
 
         .. versionadded:: 8.0.12
@@ -589,7 +639,7 @@ class Column(object):
         return self._original_name or self._name
 
     @property
-    def column_label(self):
+    def column_label(self) -> str:
         """str: The column label.
 
         .. versionadded:: 8.0.12
@@ -597,7 +647,7 @@ class Column(object):
         return self._name or self._original_name
 
     @property
-    def type(self):
+    def type(self) -> int:
         """int: The column type.
 
         .. versionadded:: 8.0.12
@@ -605,7 +655,7 @@ class Column(object):
         return self._col_type
 
     @property
-    def length(self):
+    def length(self) -> int:
         """int. The column length.
 
         .. versionadded:: 8.0.12
@@ -613,7 +663,7 @@ class Column(object):
         return self._length
 
     @property
-    def fractional_digits(self):
+    def fractional_digits(self) -> int:
         """int: The column fractional digits.
 
         .. versionadded:: 8.0.12
@@ -621,7 +671,7 @@ class Column(object):
         return self._fractional_digits
 
     @property
-    def collation_name(self):
+    def collation_name(self) -> str:
         """str: The collation name.
 
         .. versionadded:: 8.0.12
@@ -629,14 +679,14 @@ class Column(object):
         return self._collation_name
 
     @property
-    def character_set_name(self):
+    def character_set_name(self) -> str:
         """str: The character set name.
 
         .. versionadded:: 8.0.12
         """
         return self._character_set_name
 
-    def get_schema_name(self):
+    def get_schema_name(self) -> str:
         """Returns the schema name.
 
         Returns:
@@ -644,7 +694,7 @@ class Column(object):
         """
         return self._schema
 
-    def get_table_name(self):
+    def get_table_name(self) -> str:
         """Returns the table name.
 
         Returns:
@@ -652,7 +702,7 @@ class Column(object):
         """
         return self._original_table or self._table
 
-    def get_table_label(self):
+    def get_table_label(self) -> str:
         """Returns the table label.
 
         Returns:
@@ -660,7 +710,7 @@ class Column(object):
         """
         return self._table or self._original_table
 
-    def get_column_name(self):
+    def get_column_name(self) -> str:
         """Returns the column name.
 
         Returns:
@@ -668,7 +718,7 @@ class Column(object):
         """
         return self._original_name or self._name
 
-    def get_column_label(self):
+    def get_column_label(self) -> str:
         """Returns the column label.
 
         Returns:
@@ -676,7 +726,7 @@ class Column(object):
         """
         return self._name or self._original_name
 
-    def get_proto_type(self):
+    def get_proto_type(self) -> int:
         """Returns the column proto type.
 
         Returns:
@@ -684,7 +734,7 @@ class Column(object):
         """
         return self._proto_type
 
-    def get_type(self):
+    def get_type(self) -> int:
         """Returns the column type.
 
         Returns:
@@ -692,7 +742,7 @@ class Column(object):
         """
         return self._col_type
 
-    def get_length(self):
+    def get_length(self) -> int:
         """Returns the column length.
 
         Returns:
@@ -700,7 +750,7 @@ class Column(object):
         """
         return self._length
 
-    def get_fractional_digits(self):
+    def get_fractional_digits(self) -> int:
         """Returns the column fractional digits.
 
         Returns:
@@ -708,7 +758,7 @@ class Column(object):
         """
         return self._fractional_digits
 
-    def get_collation_name(self):
+    def get_collation_name(self) -> str:
         """Returns the collation name.
 
         Returns:
@@ -716,7 +766,7 @@ class Column(object):
         """
         return self._collation_name
 
-    def get_character_set_name(self):
+    def get_character_set_name(self) -> str:
         """Returns the character set name.
 
         Returns:
@@ -724,7 +774,7 @@ class Column(object):
         """
         return self._character_set_name
 
-    def is_number_signed(self):
+    def is_number_signed(self) -> bool:
         """Returns `True` if is a number signed.
 
         Returns:
@@ -732,7 +782,7 @@ class Column(object):
         """
         return self._number_signed
 
-    def is_padded(self):
+    def is_padded(self) -> Union[bool, int]:
         """Returns `True` if is padded.
 
         Returns:
@@ -740,7 +790,7 @@ class Column(object):
         """
         return self._is_padded
 
-    def is_bytes(self):
+    def is_bytes(self) -> bool:
         """Returns `True` if is bytes.
 
         Returns:
@@ -749,32 +799,37 @@ class Column(object):
         return self._is_bytes
 
 
-class Row(object):
+class Row:
     """Represents a row element returned from a SELECT query.
 
     Args:
-        rs (mysqlx.SqlResult or mysqlx.RowResult): The result set.
+        resultset (mysqlx.SqlResult or mysqlx.RowResult): The result set.
         fields (`list`): The list of fields.
     """
-    def __init__(self, rs, fields):
-        self._fields = fields
-        self._resultset = rs
 
-    def __getitem__(self, index):
+    def __init__(
+        self, resultset: Union[BufferingResult, RowResult], fields: Sequence[FieldTypes]
+    ) -> None:
+        self._fields: Sequence[FieldTypes] = fields
+        self._resultset: Union[BufferingResult, RowResult] = resultset
+
+    def __repr__(self) -> str:
+        return repr(self._fields)
+
+    def __getitem__(self, index: Union[int, str]) -> Any:
         """Returns the value of a column by name or index.
 
         .. versionchanged:: 8.0.12
         """
-        int_index = self._resultset.index_of(index) \
-            if isinstance(index, STRING_TYPES) else index
-        if int_index == -1 and isinstance(index, STRING_TYPES):
-            raise ValueError("Column name '{0}' not found".format(index))
-        elif int_index >= len(self._fields) or int_index < 0:
+        int_index = self._resultset.index_of(index) if isinstance(index, str) else index
+        if int_index == -1 and isinstance(index, str):
+            raise ValueError(f"Column name '{index}' not found")
+        if int_index >= len(self._fields) or int_index < 0:
             raise IndexError("Index out of range")
         return self._fields[int_index]
 
     @deprecated("8.0.12")
-    def get_string(self, str_index):
+    def get_string(self, str_index: str) -> str:
         """Returns the value using the column name.
 
         Args:
@@ -786,23 +841,24 @@ class Row(object):
         if int_index >= len(self._fields):
             raise IndexError("Argument out of range")
         if int_index == -1:
-            raise ValueError("Column name '{0}' not found".format(str_index))
+            raise ValueError(f"Column name '{str_index}' not found")
         return str(self._fields[int_index])
 
 
-class BaseResult(object):
+class BaseResult:
     """Provides base functionality for result objects.
 
     Args:
         connection (mysqlx.connection.Connection): The Connection object.
     """
-    def __init__(self, connection):
-        self._connection = connection
-        self._closed = False
-        self._rows_affected = 0
-        self._generated_id = -1
-        self._generated_ids = []
-        self._warnings = []
+
+    def __init__(self, connection: ConnectionType) -> None:
+        self._connection: ConnectionType = connection
+        self._closed: bool = False
+        self._rows_affected: int = 0
+        self._generated_id: int = -1
+        self._generated_ids: List[int] = []
+        self._warnings: List[Dict[str, Union[int, str]]] = []
 
         if connection is None:
             self._protocol = None
@@ -810,7 +866,7 @@ class BaseResult(object):
             self._protocol = connection.protocol
             connection.fetch_active_result()
 
-    def get_affected_items_count(self):
+    def get_affected_items_count(self) -> int:
         """Returns the number of affected items for the last operation.
 
         Returns:
@@ -818,7 +874,7 @@ class BaseResult(object):
         """
         return self._rows_affected
 
-    def get_warnings(self):
+    def get_warnings(self) -> List[Dict[str, Union[int, str]]]:
         """Returns the warnings.
 
         Returns:
@@ -826,7 +882,7 @@ class BaseResult(object):
         """
         return self._warnings
 
-    def get_warnings_count(self):
+    def get_warnings_count(self) -> int:
         """Returns the number of warnings.
 
         Returns:
@@ -834,12 +890,11 @@ class BaseResult(object):
         """
         return len(self._warnings)
 
-    def set_closed(self, flag):
-        """Sets if resultset fetch is done.
-        """
+    def set_closed(self, flag: bool) -> None:
+        """Sets if resultset fetch is done."""
         self._closed = flag
 
-    def append_warning(self, level, code, msg):
+    def append_warning(self, level: int, code: int, msg: str) -> None:
         """Append a warning.
 
         Args:
@@ -849,19 +904,16 @@ class BaseResult(object):
         """
         self._warnings.append({"level": level, "code": code, "msg": msg})
 
-    def set_generated_ids(self, generated_ids):
-        """Sets the generated ids.
-        """
+    def set_generated_ids(self, generated_ids: List[int]) -> None:
+        """Sets the generated ids."""
         self._generated_ids = generated_ids
 
-    def set_generated_insert_id(self, generated_id):
-        """Sets the generated insert id.
-        """
+    def set_generated_insert_id(self, generated_id: int) -> None:
+        """Sets the generated insert id."""
         self._generated_id = generated_id
 
-    def set_rows_affected(self, total):
-        """Sets the number of rows affected.
-        """
+    def set_rows_affected(self, total: int) -> None:
+        """Sets the number of rows affected."""
         self._rows_affected = total
 
 
@@ -873,14 +925,19 @@ class Result(BaseResult):
         connection (mysqlx.connection.Connection): The Connection object.
                                                    ids (`list`): A list of IDs.
     """
-    def __init__(self, connection=None, ids=None):
-        super(Result, self).__init__(connection)
-        self._ids = ids
+
+    def __init__(
+        self,
+        connection: Optional[ConnectionType] = None,
+        ids: Optional[List[int]] = None,
+    ) -> None:
+        super().__init__(connection)
+        self._ids: Optional[List[int]] = ids
 
         if connection is not None:
             self._connection.close_result(self)
 
-    def get_autoincrement_value(self):
+    def get_autoincrement_value(self) -> int:
         """Returns the last insert id auto generated.
 
         Returns:
@@ -889,7 +946,7 @@ class Result(BaseResult):
         return self._generated_id
 
     @deprecated("8.0.12")
-    def get_document_id(self):
+    def get_document_id(self) -> Optional[int]:
         """Returns ID of the last document inserted into a collection.
 
         .. deprecated:: 8.0.12
@@ -899,16 +956,15 @@ class Result(BaseResult):
         return self._ids[0]
 
     @deprecated("8.0.12")
-    def get_generated_insert_id(self):
+    def get_generated_insert_id(self) -> int:
         """Returns the generated insert id.
 
         .. deprecated:: 8.0.12
         """
         return self._generated_id
 
-    def get_generated_ids(self):
-        """Returns the generated ids.
-        """
+    def get_generated_ids(self) -> List[int]:
+        """Returns the generated ids."""
         return self._generated_ids
 
 
@@ -919,35 +975,35 @@ class BufferingResult(BaseResult):
         connection (mysqlx.connection.Connection): The Connection object.
                                                    ids (`list`): A list of IDs.
     """
-    def __init__(self, connection):
-        super(BufferingResult, self).__init__(connection)
-        self._columns = []
-        self._has_data = False
-        self._has_more_results = False
-        self._items = []
-        self._page_size = 0
-        self._position = -1
+
+    def __init__(self, connection: ConnectionType) -> None:
+        super().__init__(connection)
+        self._columns: List[Column] = []
+        self._has_data: bool = False
+        self._has_more_results: bool = False
+        self._items: List[Union[Row, DbDoc]] = []
+        self._page_size: int = 0
+        self._position: int = -1
         self._init_result()
 
-    def __getitem__(self, index):
+    def __getitem__(self, index: int) -> Union[Row, DbDoc]:
         return self._items[index]
 
     @property
-    def count(self):
+    def count(self) -> int:
         """int: The total of items."""
         return len(self._items)
 
-    def _init_result(self):
+    def _init_result(self) -> None:
         """Initialize the result."""
         self._columns = self._connection.get_column_metadata(self)
-        self._has_more_data = True if len(self._columns) > 0 else False
+        self._has_more_data = len(self._columns) > 0
         self._items = []
         self._page_size = 20
         self._position = -1
-        self._connection.set_active_result(self if self._has_more_data
-                                           else None)
+        self._connection.set_active_result(self if self._has_more_data else None)
 
-    def _read_item(self, dumping):
+    def _read_item(self, dumping: bool) -> Optional[Union[Row, DbDoc]]:
         """Read item.
 
         Args:
@@ -962,12 +1018,11 @@ class BufferingResult(BaseResult):
         item = [None] * len(row["field"])
         if not dumping:
             for key in range(len(row["field"])):
-                col = self._columns[key]
-                item[key] = from_protobuf(col.get_proto_type(),
-                                          row["field"][key])
+                column = self._columns[key]
+                item[key] = from_protobuf(column, row["field"][key])
         return Row(self, item)
 
-    def _page_in_items(self):
+    def _page_in_items(self) -> Union[bool, int]:
         """Reads the page items.
 
         Returns:
@@ -985,7 +1040,7 @@ class BufferingResult(BaseResult):
             count += 1
         return count
 
-    def index_of(self, col_name):
+    def index_of(self, col_name: str) -> int:
         """Returns the index of the column.
 
         Returns:
@@ -998,7 +1053,7 @@ class BufferingResult(BaseResult):
             index += 1
         return -1
 
-    def fetch_one(self):
+    def fetch_one(self) -> Optional[Union[Row, DbDoc]]:
         """Fetch one item.
 
         Returns:
@@ -1009,7 +1064,7 @@ class BufferingResult(BaseResult):
 
         return self._read_item(False)
 
-    def fetch_all(self):
+    def fetch_all(self) -> List[Union[Row, DbDoc]]:
         """Fetch all items.
 
         Returns:
@@ -1021,7 +1076,7 @@ class BufferingResult(BaseResult):
                 break
         return self._items
 
-    def set_has_data(self, flag):
+    def set_has_data(self, flag: bool) -> None:
         """Sets if result has data.
 
         Args:
@@ -1029,7 +1084,7 @@ class BufferingResult(BaseResult):
         """
         self._has_data = flag
 
-    def set_has_more_results(self, flag):
+    def set_has_more_results(self, flag: bool) -> None:
         """Sets if has more results.
 
         Args:
@@ -1044,15 +1099,13 @@ class RowResult(BufferingResult):
     Args:
         connection (mysqlx.connection.Connection): The Connection object.
     """
-    def __init__(self, connection):
-        super(RowResult, self).__init__(connection)
 
     @property
-    def columns(self):
+    def columns(self) -> List[Column]:
         """`list`: The list of columns."""
         return self._columns
 
-    def get_columns(self):
+    def get_columns(self) -> List[Column]:
         """Returns the list of columns.
 
         Returns:
@@ -1069,10 +1122,8 @@ class SqlResult(RowResult):
     Args:
         connection (mysqlx.connection.Connection): The Connection object.
     """
-    def __init__(self, connection):
-        super(SqlResult, self).__init__(connection)
 
-    def get_autoincrement_value(self):
+    def get_autoincrement_value(self) -> int:
         """Returns the identifier for the last record inserted.
 
         Returns:
@@ -1080,7 +1131,7 @@ class SqlResult(RowResult):
         """
         return self._generated_id
 
-    def next_result(self):
+    def next_result(self) -> bool:
         """Process the next result.
 
         Returns:
@@ -1092,7 +1143,7 @@ class SqlResult(RowResult):
         self._init_result()
         return True
 
-    def has_data(self):
+    def has_data(self) -> bool:
         """Returns True if result has data.
 
         Returns:
@@ -1102,6 +1153,7 @@ class SqlResult(RowResult):
         """
         return self._has_data
 
+
 class DocResult(BufferingResult):
     """Allows traversing the DbDoc objects returned by a Collection.find
     operation.
@@ -1109,10 +1161,8 @@ class DocResult(BufferingResult):
     Args:
         connection (mysqlx.connection.Connection): The Connection object.
     """
-    def __init__(self, connection):
-        super(DocResult, self).__init__(connection)
 
-    def _read_item(self, dumping):
+    def _read_item(self, dumping: bool) -> DbDoc:
         """Read item.
 
         Args:
@@ -1121,7 +1171,7 @@ class DocResult(BufferingResult):
         Returns:
             :class:`mysqlx.DbDoc`: A `DbDoc` object.
         """
-        row = super(DocResult, self)._read_item(dumping)
+        row = super()._read_item(dumping)
         if row is None:
             return None
-        return DbDoc(decode_from_bytes(row[0]))
+        return DbDoc(decode_from_bytes(row[0]))  # type: ignore[index]
